# -*- coding: utf-8 -*-


class PseudoTypeMixin:
    """
    Mixin class to serve as a base for creation of Pseudo class types.
    
    Classes derived from this class can implement custom methods depending
    on the i18n_type. Those custom method names must match the values
    available for i18n_type (settings.I18N_METHODS.keys()) in lower case
    and with an underscore in front of it.
    """
    def __init__(self, i18n_type):
        self.method_name = '_%s' % i18n_type.lower()

        # Declare method naming it accordingly to the i18n_type
        if not hasattr(self, self.method_name):
            setattr(self, self.method_name, self._base_compile)

    def _base_compile(self, string):
        raise NotImplementedError("Must be implemented in the child class.")

    # Should not be overridden
    def compile(self, string):
        """Run the correct method depending on the i18n_type."""
        return getattr(self, self.method_name)(string)  # .replace('\\\\', '\\')

    def _skip_char_around(self, string, char='\n'):
        """
        Custom pseudo method for skipping a given char around a string.
        
        The default char to be skipped is the new line (\n) one.
        
        Example:
            '\nHello\n' would call ``_base_compile`` with 'Hello' only.
        """
        starts, ends = '', ''
        n = len(char)
        if string.startswith(char):
            starts = string[:n]
            string = string[n:]
        if string.endswith(char):
            ends = string[-n:]
            string = string[:-n]
        string = self._base_compile(string)
        if starts:
            string = starts + string
        if ends:
            string = string + ends
        return string



# -*- coding: utf-8 -*-
import re
from string import Formatter

from polib import unescape


class ValidationError(Exception):
    pass


class BaseValidator(object):
    """Base class for validators.

    Implements the decorator pattern.
    """

    def __init__(self, source_language=None, target_language=None, rule=None):
        self.slang = source_language
        self.tlang = target_language
        self.rule = rule

    def __call__(self, old, new):
        """Validate the `new` translation against the `old` one.

        No checks are needed for deleted translations

        Args:
            old: The old translation.
            new: The new translation.
        Raises:
            A ValidationError with an appropriate message.
        """
        if not new or not self.precondition():
            return
        self.validate(old, new)

    def precondition(self):
        """Check whether this validator is applicable to the situation."""
        return True

    def validate(self, old, new):
        """Actual validation method.

        Subclasses must override this method.

        Args:
            old: The old translation.
            new: The new translation.
        Raises:
            A ValidationError with an appropriate message.
        """
        pass


class PrintfValidator(BaseValidator):
    """Validator that checks that the number of printf formats specifiers
    is the same in the translation.

    This is valid only if the plurals in the two languages are the same.
    """

    printf_re = re.compile(
        '%((?:(?P<ord>\d+)\$|\((?P<key>\w+)\))?(?P<fullvar>[+#-]*(?:\d+)?'\
            '(?:\.\d+)?(hh\|h\|l\|ll)?(?P<type>[\w%])))'
    )

    def precondition(self):
        """Check if the number of plurals in the two languages is the same."""
        return self.tlang.nplurals == self.slang.nplurals and \
                super(PrintfValidator, self).precondition()

    def validate(self, old, new):
        old = unescape(old)
        new = unescape(new)
        old_matches = list(self.printf_re.finditer(old))
        new_matches = list(self.printf_re.finditer(new))
        if len(old_matches) != len(new_matches):
            raise ValidationError("The number of arguments seems to differ "
                                  "between the source string and the translation."
                )


def next_splitter_or_func(string, splitters, func, pseudo_type):
    """
    Helper for doing the next splitter check.
    
    If the list is not empty, call the next splitter decorator appropriately,
    otherwise call the decorated function.
    """
    if splitters:
        return splitters[0](string, splitters[1:])(func)(pseudo_type,
            string)
    else:
        return func(pseudo_type, string)


class SplitterDecorators(object):
    """
    A class decorator that receives a list of splitter decorator classes and
    calls the first splitter from the list passing the decorated function as
    an argument as well as the list of splitters without the called splitter.
    
    In case the list of splitters is empty, it calls the decorated function
    right away.
    
    This decorator must be only used with method of classes inheriting from
    ``transifex.resources.formats.pseudo.PseudoTypeMixin``.
    """
    def __init__(self, splitters):
        self.splitters = splitters
    
    def __call__(self, func):
        def _wrapper(pseudo_type, string):
            return next_splitter_or_func(string, self.splitters, func,
                pseudo_type)
        return _wrapper


class BaseSplitter(object):
    """
    Base class decorator for splitting a given string based on a regex and
    call the subsequent splitter class available in the ``splitters`` var or
    the decorated method.
    """
    REGEX = r''

    def __init__(self, string, splitters):
        self.string = string
        self.splitters = splitters

    def __call__(self, func):
        def _wrapped(pseudo_type, string, **kwargs):
            text = []
            keys = [l.group() for l in self._regex_matches(string)]
            nkeys = len(keys)
            i = 0
            for key in keys:
                t = string.split(key, 1)
                string = t[0]
                string = next_splitter_or_func(string, self.splitters,
                    func, pseudo_type)
                text.extend([string, key])
                i += 1
                string = t[1]
            string = next_splitter_or_func(string, self.splitters,
                func, pseudo_type)
            text.append(string)
            return "".join(text)
        return _wrapped

    @classmethod
    def _regex_matches(cls, string):
        return re.finditer(cls.REGEX, string)


class PrintfSplitter(BaseSplitter):
    """
    Split the string on printf placeholders, such as %s, %d, %i, %(foo)s, etc.
    """
    # Lets reuse the printf regex from the validators
    REGEX = PrintfValidator.printf_re


class TagSplitter(BaseSplitter):
    """
    Split the string on XML/HTML tags, such as <b>, </b>, <a href="">, etc.
    """
    REGEX = r'(<|&lt;)(.|\n)*?(>|&gt;)'


class EscapedCharsSplitter(BaseSplitter):
    """
    Split the string on escaped chars, such as \\\\n, \\\\t, etc.
    """
    REGEX = r'(\\\\[\w]{1})'


class HTMLSpecialEntitiesSplitter(BaseSplitter):
    """
    Splits the string on HTML special entities, such as &lt;, &amp;, etc.
    """
    REGEX = r'&[a-zA-Z]+;'


class BraceFormatSplitter(BaseSplitter):
    """
    Splits new-style Python format strings, like {foo}, {}, and {0!r:{1}}.
    """
    def __call__(self, func):
        def _wrapped(pseudo_type, string, **kwargs):
            text = []
            formatter = Formatter()
            for (literal_text, field_name, format_spec, conversion) in formatter.parse(string):
                if literal_text:
                    literal_text = next_splitter_or_func(
                        literal_text, self.splitters, func, pseudo_type)
                    literal_text = literal_text.replace('{', '{{').replace('}', '}}')
                    text.append(literal_text)
                if field_name is not None:
                    fmt = field_name
                    if conversion is not None:
                        fmt += '!' + conversion
                    if format_spec:
                        fmt += ':' + format_spec
                    text.append('{%s}' % (fmt, ))
            return "".join(text)
        return _wrapped



# -*- coding: utf-8 -*-
import argparse
import os
import polib

from .pseudo import types

PSEUDO_TYPE_CLASSES = {
    "brackets": types.BracketsPseudoType,
    "unicode": types.UnicodePseudoType,
    "planguage": types.PLanguagePseudoType,
    "extend": types.ExtendPseudoType,
    "mixed": types.MixedPseudoTypes
}


def translate(infile, outfile, type="mixed"):
    po = polib.pofile(infile)

    # Our fake language doesn't have any special plural forms, so remove
    # the placeholder.
    if po.metadata.get('Plural-Forms') == 'nplurals=INTEGER; plural=EXPRESSION;':
        del po.metadata['Plural-Forms']

    translator = PSEUDO_TYPE_CLASSES[type]("po")

    for entry in po:
        if entry.msgid_plural:
                entry.msgstr_plural = {0: entry.msgstr_plural[0] or translator.compile(entry.msgid),
                                       1: entry.msgstr_plural[1] or translator.compile(entry.msgid_plural)}
        else:
            if not entry.msgstr:
                entry.msgstr = translator.compile(entry.msgid)

    po.save(outfile)


def main():
    parser = argparse.ArgumentParser(description="Create pseudo translation files.")
    parser.add_argument("infile", metavar="infile", help="the path to the source file")
    parser.add_argument("outfile", metavar="outfile", help="The path to save the psuedo translation to. Defaults to infile.", nargs='?')
    parser.add_argument("--type", dest="type", default="mixed", choices=PSEUDO_TYPE_CLASSES.keys(), help="The type of psuedo translation")

    args = parser.parse_args()

    translate(args.infile, args.outfile or args.infile, type=args.type)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import glossolalia

install_requires = [
    "polib",
]

setup(
    name="glossolalia",
    version=glossolalia.__version__,
    author="Fusionbox",
    author_email="programmers@fusionbox.com",
    url="https://github.com/fusionbox/glossolalia",
    description="Translation Utility to Create Pseudo Translations of PO Files",
    long_description=open('README.rst').read(),
    license=open("LICENSE").read(),
    package_data={"": ["LICENSE"]},
    packages=find_packages(exclude=('tests',)),
    zip_safe=False,
    install_requires=install_requires,
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "glossolalia = glossolalia.__main__:main",
        ],
    },
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
    ],
)
